from pathlib import Path
from typing import List

from rich.syntax import Syntax

from scriptz import cli, console, Manager


@cli.command()
def view(files: List[Path]):
    """ View files content with highlighting. """
    output_file(files, force_plain=False)


@cli.command()
def less(files: List[Path]):
    """ Page file content (no highlighting). """
    with console.pager():
        output_file(files, force_plain=True)


def output_file(files: List[Path], force_plain: bool):
    manager = Manager()

    for path in files:
        if path.is_file():
            pygment = None
            if not force_plain:
                handler = manager.config.get_handler(path)
                pygment = handler and handler.pygment
            pygment = pygment or "plain"

            content = path.read_text().strip()
            syntax = Syntax(content, lexer_name=pygment, line_numbers=True)

            console.print(path)
            console.print(syntax)
            console.print()



from .__version__ import __version__

from .cli import cli, console
from .logger import logger

from .manager import Config
from .manager import Manager

from . import commands

__all__ = (
    "__version__",
    "Config",
    "Manager",
    "cli",
    "commands",
    "console",
    "logger",
)



from typing import List, Iterator
from pathlib import Path
import sys

from scriptz import console
from . import Config, Folder, Script


class Manager:
    def __init__(self, root_dir: Path = None):
        self.root_dir = root_dir or Path.cwd()
        self.config = Config(root_dir=self.root_dir)

    @property
    def ini_path(self) -> Path:
        return self.config.ini_path

    def get_folder(self, path=None) -> Folder:
        if path is None:
            path = self.root_dir
        elif isinstance(path, str):
            path = Path(self.root_dir / path)
        return Folder(manager=self, path=path)

    def get_script(self, path: Path) -> Script:
        return Script(manager=self, path=path)

    def iterate_scripts(self, paths: List[Path]) -> Iterator[Script]:
        for path in paths:
            path = path.absolute()
            if path.is_dir():
                folder = self.get_folder(path=path)
                yield from folder.iterate_scripts()
            else:
                yield Script(manager=self, path=path)

    @classmethod
    def check(
        cls,
        check_value,
        error_msg: str,
        error_code: int = 1,
        ok_msg: str = None,
    ):
        if not check_value:
            cls.exit(error_msg=error_msg, error_code=error_code)

        elif ok_msg:
            console.print(f"[green]{ok_msg}[/green]")

    @classmethod
    def exit(cls, error_msg: str = None, error_code: int = 1):
        if error_msg:
            console.print(f"[red]{error_msg}[/red]")
        sys.exit(error_code)



from configparser import ConfigParser
import pygments.lexers


class Handler:
    def __init__(self, ini: ConfigParser, suffix: str):
        self.suffix = suffix
        self.command = ini.get(suffix, "command")
        self.comment = ini.get(suffix, "comment", fallback="#")
        self.display = ini.get(suffix, "display", fallback=self.command)
        self.options = ini.get(suffix, "options", fallback="").split()
        self.pygment = ini.get(suffix, "pygment", fallback=self.command)

        if self.pygment is None:
            lexer = pygments.lexers.get_lexer_for_filename(suffix)
            self.pygment = lexer and lexer.name

    def __eq__(self, other: str):
        return other in {self.suffix, self.command, self.display, self.pygment}

    def __str__(self):
        return self.command

    @classmethod
    def create_handlers(cls, ini: ConfigParser):
        handlers = {}

        for suffix in ini.keys():
            if suffix.startswith("."):
                handler = Handler(ini, suffix)
                handlers[handler.suffix] = handler
                handlers[handler.command] = handler
                handlers[handler.display] = handler

        return handlers



from pathlib import PurePath
from configparser import ConfigParser


class Environ:
    def __init__(self, env_ini: ConfigParser):
        self.env_ini = env_ini

    def get(self, path):
        path = PurePath(path)
        environ = dict(self.env_ini.defaults()).copy()
        for section in self.env_ini.sections():
            if path.match(section):
                for option in self.env_ini.options(section):
                    environ[option] = self.env_ini.get(section, option)
        return environ
